import { refreshToken, is_expired } from "play-dl";
import { AudioPlayerStatus, createAudioPlayer, getVoiceConnection, } from "@discordjs/voice";
import { ActionRowBuilder, StringSelectMenuBuilder, } from "discord.js";
import { validateURL, 
// addSpotify,
clearComponents, sendMessage, playCurrent, addPlaylist, } from "../utils/index.js";
import { VoiceManager } from "./voiceManager.js";
import { QueueManager } from "./queueManager.js";
export class Music {
    queueManager;
    voiceManager;
    constructor(queue) {
        if (is_expired()) {
            refreshToken();
        }
        this.queueManager = new QueueManager(queue);
        this.voiceManager = new VoiceManager();
    }
    async connect(interaction) {
        const voice_channel = interaction.member.voice.channel;
        if (!voice_channel) {
            await interaction.followUp({
                content: "You need to be in a voice channel to use this command",
                ephemeral: true,
            });
            return;
        }
        if (!voice_channel.joinable) {
            await interaction.followUp("Bot can't join this voice channel");
            return;
        }
        this.voiceManager.join({
            channelId: voice_channel.id,
            guildId: interaction.guildId,
            adapterCreator: interaction.guild.voiceAdapterCreator,
        });
    }
    async add(url, interaction) {
        const server_queue = this.queueManager.getQueue(interaction.guildId);
        const result = await validateURL(url, server_queue);
        if (result !== "video" && result !== "playlist") {
            await interaction.followUp(result);
            return;
        }
        if (result === "playlist") {
            await addPlaylist(url, server_queue, interaction);
        }
        // if (result === "spotify") {
        //   await addSpotify(url, server_queue);
        // }
        if (result === "video") {
            server_queue.queue.push(url);
        }
        server_queue.history.push(url);
        const playerStatus = this.voiceManager.getPlayer(interaction.guildId)?.state
            .status;
        if (playerStatus && playerStatus !== AudioPlayerStatus.Idle) {
            const urlType = result === "playlist" ? "playlist" : "video";
            await this.enqueued(url, interaction, urlType);
        }
        else {
            await this.play(interaction);
        }
    }
    async play(interaction) {
        const server_queue = this.queueManager.getQueue(interaction.guildId);
        const connection = getVoiceConnection(interaction.guildId);
        if (!connection) {
            await interaction.followUp("Bot is not connected to any voice channel");
            return;
        }
        const player = createAudioPlayer();
        try {
            await playCurrent(interaction, server_queue, connection, player);
        }
        catch (error) {
            const errorMessage = `Error while playing music. Please try again later.`;
            await sendMessage(interaction.channel, errorMessage);
            this.queueManager.deleteQueue(interaction.guildId);
        }
        if (!player || player.eventNames().includes(AudioPlayerStatus.Idle))
            return;
        player.on(AudioPlayerStatus.Idle, async () => {
            await this.playNext(interaction);
        });
    }
    async playNext(interaction) {
        const server_queue = this.queueManager.getQueue(interaction.guildId);
        server_queue.removeFirst();
        try {
            await Promise.all([
                clearComponents(interaction.guild, server_queue.queue_message.channelId, server_queue.queue_message.message),
                clearComponents(interaction.guild, server_queue.playing.channelId, server_queue.playing.message),
            ]);
            if (server_queue.queue.length > 0) {
                await this.play(interaction);
            }
            else {
                await sendMessage(interaction.channel, "✨ Queue is empty");
                server_queue.clearAll();
                server_queue.dispatcher = setTimeout(() => {
                    const connection = getVoiceConnection(interaction.guildId);
                    connection?.destroy();
                }, 6000);
            }
        }
        catch (error) {
            const errorMessage = `Error while playing next music. Please try again later.`;
            await sendMessage(interaction.channel, errorMessage);
            this.queueManager.deleteQueue(interaction.guildId);
        }
    }
    async enqueued(url, interaction, urlType = "video") {
        const server_queue = this.queueManager.getQueue(interaction.guildId);
        await clearComponents(interaction.guild, server_queue.queue_message.channelId, server_queue.queue_message.message);
        if (server_queue.queue.length >= 25) {
            if (urlType === "video") {
                await interaction.followUp(`Added ${url} to queue in position \`${server_queue.queue.length}\``);
            }
            return;
        }
        const row = new ActionRowBuilder();
        const option = server_queue.videoInfo.map((item, index) => {
            if (index === 0) {
                return {
                    label: `${item.title.substring(0, 25)}`,
                    value: `${index}`,
                    description: "Now Playing",
                    emoji: "🎶",
                };
            }
            else {
                return {
                    label: `${item.title.substring(0, 25)}`,
                    value: `${index}`,
                };
            }
        });
        row.addComponents(new StringSelectMenuBuilder()
            .setPlaceholder("Select a song to play")
            .setMaxValues(1)
            .setCustomId("queue")
            .addOptions(option));
        const message = await interaction.followUp({
            content: `Added ${url} to queue in position \`${server_queue.queue.length}\``,
            components: [row],
        });
        server_queue.updateQueueMessage(message.id, interaction.channelId);
    }
}
